==> ember-1.5.tar.gz: ember-1.5/src/w3e/w3e.c <==
/*
** Copyright (c) 2001  Hughes Technologies Pty Ltd.  All rights
** reserved.  
**
** Terms under which this software may be used or copied are
** provided in the  specific license associated with this product.
**
** Hughes Technologies disclaims all warranties with regard to this 
** software, including all implied warranties of merchantability and 
** fitness, in no event shall Hughes Technologies be liable for any 
** special, indirect or consequential damages or any damages whatsoever 
** resulting from loss of use, data or profits, whether in an action of 
** contract, negligence or other tortious action, arising out of or in 
** connection with the use or performance of this software.
**
**
** $Id: w3e.c,v 1.9 2002/08/26 05:04:06 bambi Exp $
**
*/

/*
** Module	: 
** Purpose	: 
** Exports	: 
** Depends Upon	: 
*/



/**************************************************************************
** STANDARD INCLUDES
**************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>


#include "config/config.h"
#include "libember/ember.h"
#include "libember/config.h"


#ifdef HAVE_UNISTD_H
#  include <unistd.h>
#endif
#ifdef HAVE_STRING_H
#  include <string.h>
#endif


/**************************************************************************
** GLOBAL VARIABLES
**************************************************************************/


/**************************************************************************
** PRIVATE ROUTINES
**************************************************************************/

static void usage()
{
	printf("\nUsage : w3-ember [-d] [ -f filename]\n\n");
	printf("	-d  Dump internal representation and exit\n");
	printf("	-f  Use the file name rather than the CGI environment\n");
	printf("	If a filename isn't given the CGI environment variables are used\n");
	printf("\n");
	exit(1);
}

static int setupScript(script, argv, argc, skip)
	ember	*script;
	char	**argv;
	int	argc;
{
	e_stab	*sym;
	e_value	val;
	int	count;

	sym = eSymCreateVariable(script,"$argc");
	eSymSetIntValue(script, &sym->value, argc - skip);

	count = 0;
	sym = eSymCreateVariable(script,"$argv");
	while(count + skip < argc)
	{
		eSymSetTextValue(script, &val, argv[count + skip]);
		eSymSetArrayElement(script, &sym->value, &val, count);
		count++;
	}
}



static void htmlCleanup(script)
	ember	*script;
{
	if (!script)
		return;
        if (script->www.parseHtml && script->www.headersSent == 0)
        {
                eWwwSendHeaders(script);
        }
        printf("</h1></h2></h3></h4></h5></h6>\n");
        printf("</select></ul></dl></ol></table>\n");
}




static void createArgVariable(script, var, val)
	ember	*script;
	char	*var,
		*val;
{
	e_stab	*sym;
	e_value	tmpVal;
	int	count;

	sym = eSymGetVariableByName(script, var);
	if (sym)
	{
		/* 
		** Hmmm, it's already there!  Must be
		** data from a multi-select widget.
		** Turn it into an array.
		*/
		if (sym->value.type != EMBER_TYPE_ARRAY)
		{
			eSymSetTextValue(script, &tmpVal, 
				sym->value.value.textVal);
			eSymFreeValue(script,&sym->value);
			eSymSetArrayElement(script, &sym->value, &tmpVal, 0);
		}
		count = eSymGetNumArrayElements(script,&sym->value);
       		eSymSetTextValue(script, &tmpVal, val);
		eSymSetArrayElement(script, &sym->value, &tmpVal, count);
	}
	else
	{
		sym = eSymCreateVariable(script, var);
       		eSymSetTextValue(script, &sym->value, val);
	}
}

void storeArgs(script, query)
	ember	*script;
        char    *query;
{
        char    *cp,
		*cp2,
                var[50],
                *val,
		*tmpVal;
	int	count;


        if (!query)
                return;

	cp = query;
	cp2 = var;
        bzero(var,sizeof(var));
	*cp2++ = '$';
	val = NULL;
        while(*cp)
        {
                if (*cp == '=')
                {
                        cp++;
			*cp2 = 0;
                        val = cp;
                        continue;
                }
                if (*cp == '&')
                {
			*cp = 0;
			tmpVal = eWwwUnEscape(val);
			createArgVariable(script, var, tmpVal);
                        cp++;
                        cp2 = var;
			*cp2++ = '$';
			val = NULL;
                        continue;
                }
		if (val)
		{
			cp++;
		}
		else
		{
                	*cp2 = *cp++;
			if (*cp2 == '.')
			{
				strcpy(cp2,"_dot_");
				cp2 += 5;
			}
			else
			{
				cp2++;
			}
		}
        }
	*cp = 0;
	tmpVal = eWwwUnEscape(val);
	createArgVariable(script, var, tmpVal);
}

#ifdef NOTDEF


/* Added by Jeff Staege April, 1998 (jstaege@thor.ece.umn.edu)
   return in buffer the next string up to token from stdin and remove token from
   stdin returns the number of characters read from stdin */
/* Modified by Georg Horn <horn@uni-koblenz.de> July 1999
   Rewrote (decomplicated) the code so that i could understand it.
   Found out that the size of 'boundary' below was to small. */

int scanToNextToken(buffer, token, maxbufsize)
char *buffer;
char *token;
int maxbufsize;
{
    char *bufp = buffer;  /* point at current position in buffer */
    char *tokp = token;  /* point at current match position in token */
    char inChar;
    int out = 0, bufsize = 0;

    /* end if you run out of input, you get a match,
       or you run out of buffer space */
    while ((inChar = getchar()) != EOF && bufsize < maxbufsize - 1) {
	out++;
	if (inChar != *tokp) {
	    /* current input char doesn't match current token char */
	    if (tokp != token) {
		/* end partial match */
		char *tp = token;
		while (tp != tokp && bufsize < (maxbufsize - 1)) {
		    *bufp++ = *tp++;
		    bufsize++;
		}
		/* reset matching pointer */
		tokp = token;
	    }
	    /* append this character to buffer */
	    *bufp++ = inChar;
	    bufsize++;
	} else {
	    /* this character is next in token */
	    /* if found full token, quit */
	    if (tokp - token + 1 >= strlen(token)) {
		break;
	    }
	    tokp++;	/* increment matching pointer */
	}
    }
    *bufp = '\0';	/* finish off buffer with end string */
    return out;
}



int saveToNextToken(file, token, maxsize)
int file;
char *token;
int maxsize;
{
    char *tokp = token;
    int out = 0, inChar, lastChar = EOF, lastlastChar = EOF;

    while ((inChar = getchar()) != EOF) {
	if (out++ >= maxsize) {
	    break;
	}
	if (inChar != *tokp) {	/* this char doesn't match token */
	    if (tokp != token) {
		/* end partial match */
		/* copy failed match to file */
		if (lastlastChar != EOF)
		    write(file, &lastlastChar, 1);
		if (lastChar != EOF)
		    write(file, &lastChar, 1);
		lastlastChar = EOF;
		lastChar = inChar;
		write(file, token, tokp - token);
		/* reset matching pointer */
		tokp = token;
		continue;
	    }
	    /* Save the pre-previous char. We have to do this because otherwise
	       we will also save the extra cr/lf that comes before the next
	       boundary string, */
	    if (lastlastChar != EOF) write(file, &lastlastChar, 1);
	    lastlastChar = lastChar;
	    lastChar = inChar;
	} else {
	    /* this char matches token */
	    /* if found token, quit */
	    if (tokp - token + 1 >= strlen(token)) {
		break;
	    }
	    tokp++;		/* increment matching pointer */
	    continue;
	}
    }
    return out;
}



void storeMultipartArgs(source)
	int source;
{
	char 	var[128], 
		val[15 * 1024], 
		boundary[128], 
		buffer[15 * 1024],
		inChar;
	int 	count, 
		contentlength, 
		file;
	sym_t 	*sym, 
		*tmp;

	/* 
	** save the length of input.  no saved file will be larger than this 
	*/
	contentlength = atoi((char *) getenv("CONTENT_LENGTH"));

	/* 
	** first line in file should be the boundary string 
	*/
	scanf("%s ", boundary);
	for(;;) 
	{
		/* scan in up to the variable name */
		if (scanToNextToken(buffer, "; name=\"", sizeof(buffer)) < 30)
			break;
		/* scan in the variable name */
		scanToNextToken(buffer, "\"", sizeof(buffer));
		strcpy(var, "$");
		strcat(var, buffer);

		/* 
		** if the variable is followed by '; filename="name"' it 
		** is a file 
		*/
		inChar = getchar();
		if (inChar == ';') 
		{
			/*
			** Get the filename and content type.  We ignore
			** the content type but have to read it anyway
			*/
			scanToNextToken(buffer, "filename=\"", sizeof(buffer));
			strcpy(val, "/tmp/");
			scanToNextToken(val + 5, "\"", sizeof(val));
			scanf(" Content-Type: %s ", buffer);

			file = open(val, O_WRONLY | O_CREAT | O_TRUNC , 0600);
			if (file < 0) 
			{
				/* 
				** can't create the file, skip the data
				*/
				strcpy(val, "");
				scanToNextToken(buffer,boundary,sizeof(buffer));
			} 
			else 
			{
				/* 
				** save the file, but don't excede the
				** provided content_length
				*/
				saveToNextToken(file, boundary, contentlength);
				close(file);
			}
		} 
		else 
		{
			/* 
			** Regular entry. Save the info
			*/
			scanf("\n");
			scanToNextToken(val, boundary, sizeof(val));
			val[strlen(val) - 2] = '\0';
		}
		/* 
		** convert var, val into symbols.  
		** This is right from storeArgs()
		*/
		sym = symGetSymbol(var);
		if (sym) 
		{
			/* 
			** Hmmm, it's already there!  Must be
			** data from a multi-select widget.
			** Turn it into an array.
			*/
			if (sym->array == SCALAR) 
			{
				sym = castToArray(sym);
			}
			count = symGetNumArrayElements(sym);
			tmp = symCreateSymbol("", TYPE_CHAR, SCALAR);
			symSetArrayElement(sym, count, tmp);
			sym = tmp;
		} 
		else 
		{
			sym = symCreateSymbol(var, TYPE_CHAR, SCALAR);
		}
		symStoreValue(sym, val, strlen(val), TYPE_CHAR);
		sym->source = source;
		sym->length = strlen(sym->val) + 1;
	}
}

#endif


void parseArgs(script)
	ember	*script;
{
        char    *query,
		*method,
		*tmpBuf,
		*content;		
	int	length,
		remain;

	method=(char *)getenv("REQUEST_METHOD");
	if (!method || strcmp(method,"GET") == 0)
	{
		query = (char *)getenv("QUERY_STRING");
		if (query)
		{
			if (*query != 0)
			{
				tmpBuf = (char *)strdup(query);
				storeArgs(script, tmpBuf);
				free(tmpBuf);
			}
		}
		return;
	}

	if (strcmp(method,"POST" )==0) 
	{
		content = (char *)getenv("CONTENT_TYPE");
		if (content && strncmp(content,"multipart/form-data",19)==0)
		{
			/* 
			** if this is multi-part, have storeMultipartArgs 
			** parse stdin and create the variable and save
			storeMultipartArgs(script);
			*/
		}			
		else 
		{	/*
			** otherwise, store stdin to a string, and send it 
			** to StoreArgs to parse it and store variables 
			*/
			length = atoi((char *)getenv("CONTENT_LENGTH"));
			query = (char *) malloc(length + 1);
			remain = length;
			while(remain)
			{
				remain -= read(fileno(stdin), 
					query + length - remain, remain);
			}
			query[length]='\0';
			storeArgs(script,query);
			free(query);
		}
	}
}



/**************************************************************************
** PUBLIC ROUTINES
**************************************************************************/

int main(argc, argv)
	int	argc;
	char	*argv[];
{
	ember	*script;
	e_stab	*symbol;
	char	c,
		*cp,
		*dir,
		*path,
		*result;
	int	usageErr = 0,
		dFlag = 0,
		fFlag = 0,
		res;


	/*
	** Check out the command line args
	*/
	if (argc > 1 && *argv[1] == '-')
	{
		while((c = getopt(argc,argv,"df")) != -1)
		{
			switch(c)
			{
				case 'd':
					if (dFlag)
						usageErr = 1;
					else
						dFlag = 1;
					break;
	
				case 'f':
					if (fFlag)
						usageErr = 1;
					else
						fFlag = 1;
					break;
	
				case '?':
					usageErr = 1;
					break;
			}
		}

		if (argc == optind < 1)
			usageErr = 1;
	}
	else
	{
		if (argc == 0)
			usageErr++;
		else
			optind = 1;
	}
	if (usageErr)
		usage();


	/*
	** Setup the given file name as the script source
	*/

	if (fFlag)
		path = argv[optind];
	else
        	path = (char *)getenv("PATH_TRANSLATED");

        if (!path)
        {
		htmlCleanup(script);
                printf("Input file name missing!\n\n");
                exit(1);
        }
	dir = strdup(path);
	cp = (char *)rindex(dir,'/');
	if (cp)
	{
		*cp = 0;
		chdir(dir);
	}

	res =  configLoadFile("w3e.conf", path, EMBER_TRUE);
	if (res < 0 && res != CONFIG_ERR_FILE)
	{
		htmlCleanup(script);
		fprintf(stdout,"Error in config file '%s'\n", 
			configGetError());
		exit(1);
	}

	script = eCreateScript();
        script->www.parseHtml = 1;
	script->useRuntimeConfig = 1;
	parseArgs(script);

	if (eFileSource(script, path) < 0)
	{
		htmlCleanup(script);
		fprintf(stdout,"Error opening file '%s'\n", 
			eGetSourceName(script));
		exit(1);
	}


	/*
	** Parse the script and report any errors
	*/
	setupScript(script, argv, argc, optind);

	if (eParseScript(script) < 0)
	{
		/* Syntax error etc */
		htmlCleanup(script);
		fprintf(stdout,"Error at line %d of script '%s'.\n",
			eGetLineNum(script), eGetSourceName(script));
		fprintf(stdout,"Error is '%s'\n\n", eGetErrorMsg(script));
		exit(1);
	}


	/*
	** If the dFlag is set then dump the symbol defs and the pcode
	** and exit
	*/
	if (dFlag)
	{
		/* Dump the pcode and symbol definitions */
		eDumpParseResult(script);
		exit(0);
	}


	/*
	** Run the script and report any errors
	*/
	if (eRunScript(script) < 0)
	{
		/* Syntax error etc */
		htmlCleanup(script);
		fprintf(stdout,"Runtime error at line %d of script '%s'.\n",
			eGetLineNum(script), eGetCurFileName(script));
		fprintf(stdout,"Error is '%s'\n\n", eGetErrorMsg(script));
		exit(1);
	}

	eFreeScript(script);
	exit(0);
}

==> ember-1.7.tar.gz: ember-1.7/src/w3e/w3e.c <==
/*
** Copyright (c) 2001  Hughes Technologies Pty Ltd.  All rights
** reserved.  
**
** Terms under which this software may be used or copied are
** provided in the  specific license associated with this product.
**
** Hughes Technologies disclaims all warranties with regard to this 
** software, including all implied warranties of merchantability and 
** fitness, in no event shall Hughes Technologies be liable for any 
** special, indirect or consequential damages or any damages whatsoever 
** resulting from loss of use, data or profits, whether in an action of 
** contract, negligence or other tortious action, arising out of or in 
** connection with the use or performance of this software.
**
**
** $Id: w3e.c,v 1.13 2002/10/03 03:22:56 bambi Exp $
**
*/

/*
** Module	: 
** Purpose	: 
** Exports	: 
** Depends Upon	: 
*/



/**************************************************************************
** STANDARD INCLUDES
**************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/file.h>


#include "config/config.h"
#include "libember/ember.h"
#include "libember/config.h"


#ifdef HAVE_UNISTD_H
#  include <unistd.h>
#endif
#ifdef HAVE_STRING_H
#  include <string.h>
#endif


/**************************************************************************
** GLOBAL VARIABLES
**************************************************************************/


/**************************************************************************
** PRIVATE ROUTINES
**************************************************************************/

static void usage()
{
	printf("\nUsage : w3-ember [-d] [ -f filename]\n\n");
	printf("	-d  Dump internal representation and exit\n");
	printf("	-f  Use the file name rather than the CGI environment\n");
	printf("	If a filename isn't given the CGI environment variables are used\n");
	printf("\n");
	exit(1);
}

static int setupScript(script, argv, argc, skip)
	ember	*script;
	char	**argv;
	int	argc;
{
	e_stab	*sym;
	e_value	val;
	int	count;

	sym = eSymCreateVariable(script,"$argc");
	eSymSetIntValue(script, &sym->value, argc - skip);

	count = 0;
	sym = eSymCreateVariable(script,"$argv");
	while(count + skip < argc)
	{
		eSymSetTextValue(script, &val, argv[count + skip]);
		eSymSetArrayElement(script, &sym->value, &val, count);
		count++;
	}
}



static void htmlCleanup(script)
	ember	*script;
{
	if (!script)
		return;
        if (script->www.parseHtml && script->www.headersSent == 0)
        {
                eWwwSendHeaders(script);
        }
        printf("</h1></h2></h3></h4></h5></h6>\n");
        printf("</select></ul></dl></ol></table>\n");
}




static void createArgVariable(script, var, val)
	ember	*script;
	char	*var,
		*val;
{
	e_stab	*sym;
	e_value	tmpVal;
	int	count;

	tmpVal.initialised = 0;
	sym = eSymGetVariableByName(script, var);
	if (sym)
	{
		/* 
		** Hmmm, it's already there!  Must be
		** data from a multi-select widget.
		** Turn it into an array.
		*/
		if (sym->value.type != EMBER_TYPE_ARRAY)
		{
			eSymSetTextValue(script, &tmpVal, 
				sym->value.value.textVal);
			eSymFreeValue(script,&sym->value);
			eSymSetArrayElement(script, &sym->value, &tmpVal, 0);
		}
		count = eSymGetNumArrayElements(script,&sym->value);
       		eSymSetTextValue(script, &tmpVal, val);
		eSymSetArrayElement(script, &sym->value, &tmpVal, count);
	}
	else
	{
		sym = eSymCreateVariable(script, var);
       		eSymSetTextValue(script, &sym->value, val);
	}
}

void storeArgs(script, query)
	ember	*script;
        char    *query;
{
        char    *cp,
		*cp2,
                var[50],
                *val,
		*tmpVal;
	int	count;


        if (!query)
                return;

	cp = query;
	cp2 = var;
        bzero(var,sizeof(var));
	*cp2++ = '$';
	val = NULL;
        while(*cp)
        {
                if (*cp == '=')
                {
                        cp++;
			*cp2 = 0;
                        val = cp;
                        continue;
                }
                if (*cp == '&')
                {
			*cp = 0;
			tmpVal = eWwwUnEscape(val);
			createArgVariable(script, var, tmpVal);
                        cp++;
                        cp2 = var;
			*cp2++ = '$';
			val = NULL;
                        continue;
                }
		if (val)
		{
			cp++;
		}
		else
		{
                	*cp2 = *cp++;
			if (*cp2 == '.')
			{
				strcpy(cp2,"_dot_");
				cp2 += 5;
			}
			else
			{
				cp2++;
			}
		}
        }
	*cp = 0;
	tmpVal = eWwwUnEscape(val);
	createArgVariable(script, var, tmpVal);
}



int scanToNextToken(buffer, token, maxbufsize)
	char 	*buffer;
	char 	*token;
	int 	maxbufsize;
{
    	char 	*bufp = buffer,  /* point at current position in buffer */
    		*tokp = token,  /* point at current match position in token */
    		inChar;
    	int 	out = 0, 
		bufsize = 0;

    	/* 
	** end if you run out of input, you get a match,
	** or you run out of buffer space 
	*/
	while ((inChar = getchar()) != EOF && bufsize < maxbufsize - 1) 
	{
		out++;
		if (inChar != *tokp) 
		{
	    		/* 
			** current input char doesn't match current token char 
			*/
	    		if (tokp != token) 
			{
				/* end partial match */
				char *tp = token;
				while (tp != tokp && bufsize < (maxbufsize - 1))
				{
		    			*bufp++ = *tp++;
		    			bufsize++;
				}
				/* reset matching pointer */
				tokp = token;
	    		}
	    		/* 
			** append this character to buffer 
			*/
	    		*bufp++ = inChar;
	    		bufsize++;
		} 
		else 
		{
	    		/* 
			** this character is next in token 
	    		** if found full token, quit 
			*/
	    		if (tokp - token + 1 >= strlen(token)) 
			{
				break;
	    		}
	    		tokp++;	/* increment matching pointer */
		}
    	}
    	*bufp = '\0';	/* finish off buffer with end string */
    	return out;
}



int saveToNextToken(file, token, maxsize)
	int 	file;
	char 	*token;
	int 	maxsize;
{
	char *tokp = token;
	int 	out = 0, 	
		inChar, 
		lastChar = EOF, 
		lastlastChar = EOF;

	while ((inChar = getchar()) != EOF) 
	{
		if (out++ >= maxsize) 
		{
	    		break;
		}
		if (inChar != *tokp) 	/* this char doesn't match token */
		{
	    		if (tokp != token) 
			{
				/* end partial match */
				/* copy failed match to file */
				if (lastlastChar != EOF)
		    			write(file, &lastlastChar, 1);
				if (lastChar != EOF)
		    			write(file, &lastChar, 1);
				lastlastChar = EOF;
				lastChar = inChar;
				write(file, token, tokp - token);
				/* reset matching pointer */
				tokp = token;
				continue;
	    		}
	    		/* 
			** Save the pre-previous char. We have to do 
			** this because otherwise we will also save the 
			** extra cr/lf that comes before the next
	       		** boundary string, 
			*/
	    		if (lastlastChar != EOF) 
				write(file, &lastlastChar, 1);
	    		lastlastChar = lastChar;
	    		lastChar = inChar;
		} 
		else 
		{
			/* this char matches token */
	    		/* if found token, quit */
	    		if (tokp - token + 1 >= strlen(token)) 
			{
				break;
	    		}
	    		tokp++;		/* increment matching pointer */
	    		continue;
		}
    	}
	return out;
}



static void setSymbolValue(script, var, val)
	ember	*script;
	char	*var,
		*val;
{
	e_stab 	*sym; 
	e_value	tmpVal;
	int	count;

	tmpVal.initialised = 0;
	sym = eSymGetVariableByName(script, var);
	if (sym)
	{
		/* 
		** Hmmm, it's already there!  Must be
		** data from a multi-select widget.
		** Turn it into an array.
		*/
		if (sym->value.type != EMBER_TYPE_ARRAY)
		{
			eSymSetTextValue(script, &tmpVal, 
				sym->value.value.textVal);
			eSymFreeValue(script,&sym->value);
			eSymSetArrayElement(script, &sym->value, 
				&tmpVal, 0);
		}
		count = eSymGetNumArrayElements(script,&sym->value);
       		eSymSetTextValue(script, &tmpVal, val);
		eSymSetArrayElement(script,&sym->value,&tmpVal,count);
	}
	else
	{
		sym = eSymCreateVariable(script, var);
       		eSymSetTextValue(script, &sym->value, val);
	}
}

void storeMultipartArgs(script, source)
	ember	*script;
	int 	source;
{
	char 	var[128], 
		srcFilename[255],
		dstFilename[255],
		val[15 * 1024], 
		boundary[128], 
		buffer[15 * 1024],
		inChar;
	int 	count, 
		contentlength, 
		file;

	/* 
	** save the length of input.  no saved file will be larger than this 
	*/
	contentlength = atoi((char *) getenv("CONTENT_LENGTH"));

	/* 
	** first line in file should be the boundary string 
	*/
	scanf("%s ", boundary);
	for(;;) 
	{
		/* scan in up to the variable name */
		if (scanToNextToken(buffer, "; name=\"", sizeof(buffer)) < 30)
			break;
		/* scan in the variable name */
		scanToNextToken(buffer, "\"", sizeof(buffer));
		strcpy(var, "$");
		strcat(var, buffer);

		/* 
		** if the variable is followed by '; filename="name"' it 
		** is a file 
		*/
		inChar = getchar();
		if (inChar == ';') 
		{
			/*
			** Get the filename and content type.  We ignore
			** the content type but have to read it anyway
			*/
			scanToNextToken(buffer, "filename=\"", sizeof(buffer));
			snprintf(dstFilename,255, "/tmp/w3e.%d.%d",getpid(),
				time(NULL));
			scanToNextToken(srcFilename, "\"", sizeof(val));
			scanf(" Content-Type: %s ", buffer);

			file = open(dstFilename, O_WRONLY | O_CREAT | O_TRUNC,
				 0600);
			if (file < 0) 
			{
				/* 
				** can't create the file, skip the data
				*/
				strcpy(val, "");
				scanToNextToken(buffer,boundary,sizeof(buffer));
				setSymbolValue(script, var, val);
			} 
			else 
			{
				/* 
				** save the file, but don't excede the
				** provided content_length
				*/
				saveToNextToken(file, boundary, contentlength);
				close(file);
				setSymbolValue(script, var, dstFilename);
				setSymbolValue(script, var, srcFilename);
			}
		} 
		else 
		{
			/* 
			** Regular entry. Save the info
			*/
			scanf("\n");
			scanToNextToken(val, boundary, sizeof(val));
			val[strlen(val) - 2] = '\0';
			setSymbolValue(script, var, val);
		}
	}
}



void parseArgs(script)
	ember	*script;
{
        char    *query,
		*method,
		*tmpBuf,
		*content;		
	int	length,
		remain;

	method=(char *)getenv("REQUEST_METHOD");
	if (!method || strcmp(method,"GET") == 0)
	{
		query = (char *)getenv("QUERY_STRING");
		if (query)
		{
			if (*query != 0)
			{
				tmpBuf = (char *)strdup(query);
				storeArgs(script, tmpBuf);
				free(tmpBuf);
			}
		}
		return;
	}

	if (strcmp(method,"POST" )==0) 
	{
		content = (char *)getenv("CONTENT_TYPE");
		if (content && strncmp(content,"multipart/form-data",19)==0)
		{
			/* 
			** if this is multi-part, have storeMultipartArgs 
			** parse stdin and create the variable and save
			*/
			storeMultipartArgs(script, script);
		}			
		else 
		{	/*
			** otherwise, store stdin to a string, and send it 
			** to StoreArgs to parse it and store variables 
			*/
			length = atoi((char *)getenv("CONTENT_LENGTH"));
			query = (char *) malloc(length + 1);
			remain = length;
			while(remain)
			{
				remain -= read(fileno(stdin), 
					query + length - remain, remain);
			}
			query[length]='\0';
			storeArgs(script,query);
			free(query);
		}
	}
}



/**************************************************************************
** PUBLIC ROUTINES
**************************************************************************/

int main(argc, argv)
	int	argc;
	char	*argv[];
{
	ember	*script;
	e_stab	*symbol;
	char	c,
		*cp,
		*dir,
		*path,
		*result;
	int	usageErr = 0,
		dFlag = 0,
		fFlag = 0,
		res;


	/*
	** Check out the command line args
	*/
	if (argc > 1 && *argv[1] == '-')
	{
		while((c = getopt(argc,argv,"df")) != -1)
		{
			switch(c)
			{
				case 'd':
					if (dFlag)
						usageErr = 1;
					else
						dFlag = 1;
					break;
	
				case 'f':
					if (fFlag)
						usageErr = 1;
					else
						fFlag = 1;
					break;
	
				case '?':
					usageErr = 1;
					break;
			}
		}

		if (argc == optind < 1)
			usageErr = 1;
	}
	else
	{
		if (argc == 0)
			usageErr++;
		else
			optind = 1;
	}
	if (usageErr)
		usage();


	/*
	** Setup the given file name as the script source
	*/

	if (fFlag)
		path = argv[optind];
	else
        	path = (char *)getenv("PATH_TRANSLATED");

        if (!path)
        {
		htmlCleanup(script);
                printf("Input file name missing!\n\n");
                exit(1);
        }
	dir = strdup(path);
	cp = (char *)rindex(dir,'/');
	if (cp)
	{
		*cp = 0;
		chdir(dir);
	}

	res =  eConfigLoadFile("w3e.conf", path, EMBER_TRUE);
	if (res < 0 && res != CONFIG_ERR_FILE)
	{
		htmlCleanup(script);
		fprintf(stdout,"Error in config file '%s'\n", 
			eConfigGetError());
		exit(1);
	}

	script = eCreateScript();
        script->www.parseHtml = 1;
	script->useRuntimeConfig = 1;
	parseArgs(script);

	if (eFileSource(script, path) < 0)
	{
		htmlCleanup(script);
		fprintf(stdout,"Error opening file '%s'\n", 
			eGetSourceName(script));
		exit(1);
	}


	/*
	** Parse the script and report any errors
	*/
	setupScript(script, argv, argc, optind);

	if (eParseScript(script) < 0)
	{
		/* Syntax error etc */
		htmlCleanup(script);
		fprintf(stdout,"Error at line %d of script '%s'.\n",
			eGetLineNum(script), eGetSourceName(script));
		fprintf(stdout,"Error is '%s'\n\n", eGetErrorMsg(script));
		exit(1);
	}


	/*
	** If the dFlag is set then dump the symbol defs and the pcode
	** and exit
	*/
	if (dFlag)
	{
		/* Dump the pcode and symbol definitions */
		eDumpParseResult(script);
		exit(0);
	}


	/*
	** Run the script and report any errors
	*/
	if (eRunScript(script) < 0)
	{
		/* Syntax error etc */
		htmlCleanup(script);
		fprintf(stdout,"Runtime error at line %d of script '%s'.\n",
			eGetLineNum(script), eGetCurFileName(script));
		fprintf(stdout,"Error is '%s'\n\n", eGetErrorMsg(script));
		exit(1);
	}

	eFreeScript(script);
	exit(0);
}
